#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "elinks.h"

#include "document/dom/ecmascript/spidermonkey.h"
#include "document/dom/ecmascript/spidermonkey/Node.h"
#include "document/dom/ecmascript/spidermonkey/html/HTMLMapElement.h"
#include "dom/node.h"

static JSBool
HTMLMapElement_getProperty(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
{
	struct dom_node *node;
	struct MAP_struct *html;

	if (!JSVAL_IS_INT(id))
		return JS_TRUE;

	if (!obj || (!JS_InstanceOf(ctx, obj, (JSClass *)&HTMLMapElement_class, NULL)))
		return JS_FALSE;

	node = JS_GetPrivate(ctx, obj);
	if (!node)
		return JS_FALSE;
	html = node->data.element.html_data;
	if (!html)
		return JS_FALSE;

	switch (JSVAL_TO_INT(id)) {
	case JSP_HTML_MAP_ELEMENT_AREAS:
		string_to_jsval(ctx, vp, html->areas);
		/* Write me! */
		break;
	case JSP_HTML_MAP_ELEMENT_NAME:
		string_to_jsval(ctx, vp, html->name);
		break;
	default:
		return HTMLElement_getProperty(ctx, obj, id, vp);
	}
	return JS_TRUE;
}

static JSBool
HTMLMapElement_setProperty(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
{
	struct dom_node *node;
	struct MAP_struct *html;

	if (!JSVAL_IS_INT(id))
		return JS_TRUE;

	if (!obj || (!JS_InstanceOf(ctx, obj, (JSClass *)&HTMLMapElement_class, NULL)))
		return JS_FALSE;

	node = JS_GetPrivate(ctx, obj);
	if (!node)
		return JS_FALSE;
	html = node->data.element.html_data;
	if (!html)
		return JS_FALSE;

	switch (JSVAL_TO_INT(id)) {
	case JSP_HTML_MAP_ELEMENT_NAME:
		mem_free_set(&html->name, stracpy(jsval_to_string(ctx, vp)));
		break;
	default:
		return HTMLElement_setProperty(ctx, obj, id, vp);
	}
	return JS_TRUE;
}

const JSPropertySpec HTMLMapElement_props[] = {
	{ "areas",	JSP_HTML_MAP_ELEMENT_AREAS,	JSPROP_ENUMERATE | JSPROP_READONLY},
	{ "name",	JSP_HTML_MAP_ELEMENT_NAME,	JSPROP_ENUMERATE },
	{ NULL }
};

const JSFunctionSpec HTMLMapElement_funcs[] = {
	{ NULL }
};

const JSClass HTMLMapElement_class = {
	"HTMLMapElement",
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub, JS_PropertyStub,
	HTMLMapElement_getProperty, HTMLMapElement_setProperty,
	JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, Node_finalize
};

void
make_MAP_object(JSContext *ctx, struct dom_node *node)
{
	struct html_objects *o = JS_GetContextPrivate(ctx);

	node->data.element.html_data = mem_calloc(1, sizeof(struct MAP_struct));
	if (node->data.element.html_data) {
		node->ecmascript_obj = JS_NewObject(ctx, (JSClass *)&HTMLMapElement_class, o->HTMLElement_object, NULL);
	}
}

void
done_MAP_object(struct dom_node *node)
{
	struct MAP_struct *d = node->data.element.html_data;

	/* areas ? */
	mem_free_if(d->name);
}
